#!/bin/sh 
IMAGENAME=$1
OUTDIR=$2
WORKDIR=`dirname $0`
WORKDIR=`cd $WORKDIR; pwd`
NAME=`echo "$IMAGENAME" | sed "s/.*\///g" | sed "s/\..*//"`

FW_SEARCH_STRING="mtd_fw.sum.bin|FW.img"
ROOT_SEARCH_STRING="mtd_root.sum.bin|ROOT.img"

FILESIZE=`ls -l $IMAGENAME | awk '{ print $5 }'`
FILECLUSTERS=$(($FILESIZE / 512 -1))
CURRENTCLUSTER=1
NAND_UIMAGE=1
NAND_FW_CLUSTER=0
NAND_ROOT_CLUSTER=0
LINE="echo >----------------------------------------------------------<"
echo "Files size: $FILESIZE ($FILECLUSTERS clusters)"

#Below calculation of clusters is just approximation - not needed, but greatly speadup the process
echo "[DBA] Analysing img file, please wait..."

if `egrep -qUbuo --binary-files=text "$FW_SEARCH_STRING" $IMAGENAME`;then
  NAND_FW_CLUSTER=$((`egrep -Ubuo --binary-files=text "$FW_SEARCH_STRING" $IMAGENAME | cut -d ":" -f1` / 512 - 10))
else
  $NAND_FW_CLUSTER = 6
  $LINE; echo "Probably something wrong!!!. No FW.img found :("
fi

if `egrep -qUbuo --binary-files=text "$ROOT_SEARCH_STRING" $IMAGENAME`;then
  NAND_ROOT_CLUSTER=$((`egrep -Ubuo --binary-files=text "$ROOT_SEARCH_STRING" $IMAGENAME | cut -d ":" -f1` / 512 - 10))
fi

$LINE; echo "[DBA] Iterating img file, please wait..."

while [ $CURRENTCLUSTER -le "$FILECLUSTERS" ]
do
	dd if=$IMAGENAME of=/tmp/tmpcluster bs=512 skip=$CURRENTCLUSTER count=1 >/dev/null 2>&1
	CURRENTCLUSTER=$(($CURRENTCLUSTER+1))
	if `egrep -qUbuo --binary-files=text 'uImage' /tmp/tmpcluster`;then
		NAND_UIMAGE=$(($CURRENTCLUSTER + 1))
		echo "Pointer to uImage: $NAND_UIMAGE cluster"
		CURRENTCLUSTER=$NAND_FW_CLUSTER # kernel for sure has more than 1,5MB ;)
	fi
	if `egrep -qUbuo --binary-files=text "$FW_SEARCH_STRING" /tmp/tmpcluster`;then
		NAND_FW_CLUSTER=$CURRENTCLUSTER
		echo "Pointer to NAND FW: $NAND_FW_CLUSTER cluster"
		CURRENTCLUSTER=$NAND_ROOT_CLUSTER # NAND FW has always 8MB ;)
	fi
	if `egrep -qUbuo --binary-files=text "$ROOT_SEARCH_STRING" /tmp/tmpcluster`;then
		NAND_ROOT_CLUSTER=$CURRENTCLUSTER
		echo "Pointer to NAND ROOT: $CURRENTCLUSTER cluster"
		break
		date
	fi
done

#extracting mtdblocks
if [ $NAND_UIMAGE -gt 0 ]; then
  #CLUSTERDIFF=$(($NAND_FW_CLUSTER - $NAND_UIMAGE - 1))
  CLUSTERDIFF=$(($NAND_FW_CLUSTER - $NAND_UIMAGE))
  CLUSTERDIFF=$(($CLUSTERDIFF*2 -1 )) # needed to remove garbage
  NAND_UIMAGE256=$(($NAND_UIMAGE*2))
  $LINE; echo "[DBA] Creating uImage from $DBAarchive , please wait..."
  dd if=$IMAGENAME of=$WORKDIR/uImage bs=256 skip=$NAND_UIMAGE256 count=$CLUSTERDIFF
fi
if [ $NAND_FW_CLUSTER -gt 0 ]; then
  #CLUSTERDIFF=$(($NAND_ROOT_CLUSTER - $NAND_FW_CLUSTER - 1))
  CLUSTERDIFF=$(($NAND_ROOT_CLUSTER - $NAND_FW_CLUSTER))
  $LINE; echo "[DBA] Creating NAND FW block file from $DBAarchive , please wait..."
  dd if=$IMAGENAME of=$WORKDIR/tmpimgfw bs=512 skip=$NAND_FW_CLUSTER count=$CLUSTERDIFF
fi
if [ $NAND_ROOT_CLUSTER -gt 0 ]; then
  $LINE; echo "[DBA] Creating NAND_ROOT block file from $DBAarchive , please wait..."
  dd if=$IMAGENAME of=$WORKDIR/tmpimgroot bs=512 skip=$NAND_ROOT_CLUSTER
fi

#cleaning-up before proceeding - just in case
umount /dev/mtdblock0 >/dev/null 2>&1
if [ -e $WORKDIR/TMPFLDR ]; then rm -rf $WORKDIR/TMPFLDR; fi
mkdir $WORKDIR/TMPFLDR
rmmod mtdblock >/dev/null 2>&1
rmmod mtdram >/dev/null 2>&1

#extracting files from jffs2's
modprobe jffs2
modprobe mtdram total_size=65536 erase_size=128
sleep 2
cat /proc/mtd
modprobe mtdblock
sleep 2 
if [ -e $WORKDIR/tmpimgroot ]; then
  $LINE; echo "[DBA] Extracting files from NAND_ROOT, please wait..."
  dd if=$WORKDIR/tmpimgroot  of=/dev/mtdblock0
  sleep 2
  mount -t jffs2 /dev/mtdblock0  $WORKDIR/TMPFLDR
  sleep 3
  mv -f $WORKDIR/uImage $WORKDIR/TMPFLDR/boot/
  find ./TMPFLDR/ -print0 | xargs -0 touch -c  >/dev/null 2>&1
  touch -c $WORKDIR/TMPFLDR/*  >/dev/null 2>&1
  cd $WORKDIR
  tar -cf $WORKDIR/rootimg.tar.gz ./TMPFLDR/*
  sleep 5
  umount /dev/mtdblock0
  rm -f $WORKDIR/tmpimgroot
fi
if [ -e $WORKDIR/tmpimgfw ]; then
  $LINE; echo "[DBA] Extracting files from NAND_FW, please wait..."
  dd if=$WORKDIR/tmpimgfw  of=/dev/mtdblock0
  sleep 2
  mount -t jffs2 /dev/mtdblock0  $WORKDIR/TMPFLDR
  sleep 3
  touch -c $WORKDIR/TMPFLDR/* >/dev/null 2>&1
  cd $WORKDIR/TMPFLDR/
  tar -cf $WORKDIR/fwimg.tar.gz *
  sleep 5
  cd $WORKDIR
  umount /dev/mtdblock0
  rm -f $WORKDIR/tmpimgfw
fi
rmmod mtdblock
rmmod mtdram
$LINE; echo "[DBA] Creating $NAME.tar.gz, please wait..."
cd $WORKDIR
tar -xf $WORKDIR/rootimg.tar.gz
tar -xf $WORKDIR/fwimg.tar.gz -C ./TMPFLDR/boot
rm -f $WORKDIR/rootimg.tar.gz
rm -f $WORKDIR/fwimg.tar.gz
#patching files = bocking mtdblocks to be mounted
if [ -f ./TMPFLDR/etc/fstab ]; then sed -i "s;^/dev/mtdblock;#/dev/mtdblock;;" ./TMPFLDR/etc/fstab; fi
if [ -f ./TMPFLDR/var/etc/fstab ]; then sed -i "s;^/dev/mtdblock;#/dev/mtdblock;;" ./TMPFLDR/var/etc/fstab; fi
sed -i "s;^mtdblock;#mtdblock;;" ./TMPFLDR/var/etc/auto.default
sed -i "s;^mtdblock;#mtdblock;;" ./TMPFLDR/var/etc/automount/auto.misc
sed -i "s;^[^#']*/dev/mtdblock3 /boot;echo 'Disabled by multiboot# &';;" ./TMPFLDR/etc/init.d/rcS
sed -i "s;mtdblock5);disabled&;;" ./TMPFLDR/sbin/hotplug.sh
sed -i "s;^[^#']*mtdlist=\"mtdblock5\";#&;;"  ./TMPFLDR/sbin/aaf-first.sh
touch ./TMPFLDR/etc/.usbimage
touch ./TMPFLDR/var/etc/.usbimage
#Last step - creation of final tar.gz file
cd $WORKDIR/TMPFLDR/
tar -czf $WORKDIR/$NAME.tar.gz *
cd $WORKDIR
#rm -rf $WORKDIR/TMPFLDR
